@@ -43,12 +43,12 @@ from . import save_load_algorithms
 
 
 
-def redefine_doc(docstr):
-    """
-    Decorator: adoption outer func.__doc__.
-    """
-    def _decorator(func):
-        func.__doc__ = docstr
+def inherit_doc(doc : str) -> Callable:
+    '''
+    Decorator: inheritance outer func.__doc__.
+    '''
+    def _decorator(func : Callable) -> Callable:
+        func.__doc__ = doc
         return func
     return _decorator
 
@@ -64,80 +64,69 @@ def redefine_doc(docstr):
 class Snaplib:
     '''
     data preprocessing library
+
+    This library has methods for preprocessig data : pandas.DataFrame
+        1. nan_info
+        2. nan_plot
+        3. cleane
+        4. recover_data - NaN imputing with ML,
+        5. train_test_split_balanced
+        6. encode_dataframe
+        7. decode_dataframe
+        8. k_folds_split
+
+    For one and list of algorithms
+        9. cross_val
+        10. features_selection_clsf
+        11. fit_stacked
+        12. save_stack
+        13. load_stack
+        14. predict_stacked
     '''
+
     ENCODER = encoder.Encoder()
 
 
 
 
     @staticmethod
-    @redefine_doc(nans.nan_info.__doc__)
-    def nan_info(df : pd.DataFrame) -> pd.DataFrame:
-        return nans.nan_info(df)
+    @inherit_doc(nans.nan_info.__doc__)
+    def nan_info(*args, **kwargs):
+        return nans.nan_info(*args, **kwargs)
+
+
+    @staticmethod
+    @inherit_doc(nans.nan_plot.__doc__)
+    def nan_plot(*args, **kwargs):
+        return nans.nan_plot(*args, **kwargs)
+
+
+    @staticmethod
+    @inherit_doc(nans.cleane.__doc__)
+    def cleane(*args, **kwargs): 
+        return nans.cleane(*args, **kwargs)
+
+
+    @inherit_doc(encoder.Encoder.encode_dataframe.__doc__)
+    def encode_dataframe(self, *args, **kwargs): 
+        return self.ENCODER.encode_dataframe(*args, **kwargs)
+
+
+    @inherit_doc(encoder.Encoder.decode_dataframe.__doc__)
+    def decode_dataframe(self,*args, **kwargs): 
+        return self.ENCODER.decode_dataframe(*args, **kwargs)
+
+
+    @staticmethod
+    @inherit_doc(splitter.k_folds_split.__doc__)
+    def k_folds_split(*args, **kwargs): 
+        return splitter.k_folds_split(*args, **kwargs)
 
 
 
 
     @staticmethod
-    @redefine_doc(nans.nan_plot.__doc__)
-    def nan_plot(df : pd.DataFrame) -> plt.figure:
-        return nans.nan_plot(df)
-
-
-
-
-    @staticmethod
-    @redefine_doc(nans.cleane.__doc__)
-    def cleane( df : pd.DataFrame, 
-                target : str = None, 
-                verbose : bool = True
-                ) -> pd.DataFrame: 
-        return nans.cleane(df, target, verbose)
-
-
-
-
-    @redefine_doc(encoder.Encoder.encode_dataframe.__doc__)
-    def encode_dataframe(self, 
-                         df : pd.DataFrame, 
-                         inplace : bool = False, 
-                         ) -> pd.DataFrame: 
-        if inplace is False:
-            df_copied = df.copy()
-            return self.ENCODER.encode_dataframe(df_copied)
-        elif inplace is True:
-            return self.ENCODER.encode_dataframe(df)
-
-
-
-    
-    @redefine_doc(encoder.Encoder.decode_dataframe.__doc__)
-    def decode_dataframe(self, 
-                         df : pd.DataFrame, 
-                         inplace : bool = False, 
-                         ) -> pd.DataFrame: 
-        if inplace is False:
-            df_copied = df.copy()
-            return self.ENCODER.decode_dataframe(df_copied)
-        elif inplace is True:
-            return self.ENCODER.decode_dataframe(df)
-
-
-
-
-    @staticmethod
-    @redefine_doc(splitter.k_folds_split.__doc__)
-    def k_folds_split(df : pd.DataFrame, 
-                      target_name : str, 
-                      k : int, 
-                      ) -> dict: 
-        return splitter.k_folds_split(df, target_name, k)
-
-
-
-
-    @staticmethod
-    @redefine_doc(splitter.train_test_split_balanced.__doc__)
+    @inherit_doc(splitter.train_test_split_balanced.__doc__)
     def train_test_split_balanced(  df : pd.DataFrame, 
                                     target_feature : str, 
                                     test_size : float = 0.2, 
@@ -155,7 +144,7 @@ class Snaplib:
 
 
     @staticmethod
-    @redefine_doc(cross_validation.cross_val.__doc__)
+    @inherit_doc(cross_validation.cross_val.__doc__)
     def cross_val(algorithms : list, 
                   k_fold_dict : dict, 
                   metric : Callable, 
@@ -178,7 +167,7 @@ class Snaplib:
 
 
     @staticmethod
-    @redefine_doc(recover.recover_data.__doc__)
+    @inherit_doc(recover.recover_data.__doc__)
     def recover_data(df_in : pd.DataFrame, 
                      device : str = 'cpu',
                      verbose : int or bool = True,
@@ -194,7 +183,7 @@ class Snaplib:
 
 
     @staticmethod
-    @redefine_doc(fit_pred_bagged.fit_stacked.__doc__)
+    @inherit_doc(fit_pred_bagged.fit_stacked.__doc__)
     def fit_stacked(algorithms_list : list, 
                     X_train : pd.DataFrame, 
                     y_train : pd.Series or np.ndarray, 
@@ -217,7 +206,7 @@ class Snaplib:
 
 
     @staticmethod
-    @redefine_doc(fit_pred_bagged.predict_stacked.__doc__)
+    @inherit_doc(fit_pred_bagged.predict_stacked.__doc__)
     def predict_stacked(algorithms_list : list, 
                         X_pred : pd.DataFrame, 
                         task : str = 'clsf'
@@ -231,7 +220,7 @@ class Snaplib:
 
 
     @staticmethod
-    @redefine_doc(feature_selection_bagged.features_selection_regr.__doc__)
+    @inherit_doc(feature_selection_bagged.features_selection_regr.__doc__)
     def features_selection_regr(algorithms : list, 
                                 df : pd.DataFrame, 
                                 target : str, 
@@ -254,7 +243,7 @@ class Snaplib:
 
 
     @staticmethod
-    @redefine_doc(feature_selection_bagged.features_selection_clsf.__doc__)
+    @inherit_doc(feature_selection_bagged.features_selection_clsf.__doc__)
     def features_selection_clsf(algorithms : list, 
                                 df : pd.DataFrame, 
                                 target : str, 
@@ -277,7 +266,7 @@ class Snaplib:
 
 
     @staticmethod
-    @redefine_doc(save_load_algorithms.save_stack.__doc__)
+    @inherit_doc(save_load_algorithms.save_stack.__doc__)
     def save_stack( algorithms_list : list, 
                     directory : str = ''
                     ) -> None:
@@ -288,7 +277,7 @@ class Snaplib:
 
 
     @staticmethod
-    @redefine_doc(save_load_algorithms.load_stack.__doc__)
+    @inherit_doc(save_load_algorithms.load_stack.__doc__)
     def load_stack( names_list : list, 
                     directory : str = ''
                     ) -> None:
